@@ -3,14 +3,14 @@
 '''
 A library to access Tryton's models like a client.
 '''
-import threading
 import datetime
 import functools
+import threading
 from decimal import Decimal
 
 import proteus.config
 
-__version__ = "6.0.8"
+__version__ = "7.0.1"
 __all__ = ['Model', 'Wizard', 'Report']
 
 _MODELS = threading.local()
@@ -145,6 +145,13 @@ class CharDescriptor(FieldDescriptor):
 
     def __set__(self, instance, value):
         assert isinstance(value, str) or value is None
+        if self.definition.get('strip') and value:
+            if self.definition['strip'] == 'leading':
+                value = value.lstrip()
+            elif self.definition['strip'] == 'trailing':
+                value = value.rstrip()
+            else:
+                value = value.strip()
         super(CharDescriptor, self).__set__(instance, value or '')
 
 
@@ -531,6 +538,7 @@ class ModelList(list):
         self.record_removed = set()
         self.record_deleted = set()
         result = super(ModelList, self).__init__(sequence)
+        assert len(self) == len(set(self))
         self.__check(self, on_change=False)
         return result
     __init__.__doc__ = list.__init__.__doc__
@@ -577,17 +585,17 @@ class ModelList(list):
 
     def append(self, record):
         self.__check([record])
-        res = super(ModelList, self).append(record)
+        if record not in self:
+            super().append(record)
         self._changed()
-        return res
     append.__doc__ = list.append.__doc__
 
     def extend(self, iterable):
         iterable = list(iterable)
         self.__check(iterable)
-        res = super(ModelList, self).extend(iterable)
+        set_ = set(self)
+        super().extend(r for r in iterable if r not in set_)
         self._changed()
-        return res
     extend.__doc__ = list.extend.__doc__
 
     def insert(self, index, record):
@@ -905,7 +913,15 @@ class Model(object):
         if change is None:
             cls.save(records)
             cls.reload(records)  # Force reload because save doesn't always
-            return getattr(proxy, button)([r.id for r in records], context)
+            record_ids = [r.id for r in records]
+            action = getattr(proxy, button)(record_ids, context)
+            if action and not isinstance(action, str):
+                if isinstance(action, int):
+                    action = config.get_proxy('ir.action').get_action_value(
+                        action, context)
+                return _convert_action(
+                    action, records, context=context, config=config)
+            return action
         else:
             record, = records
             values = record._on_change_args(change)
@@ -959,12 +975,6 @@ class Model(object):
         fields.append('_timestamp')
         self._values.update(
             self._proxy.read([self.id], fields, self._context)[0])
-        for field in fields:
-            if (field in self._fields
-                    and self._fields[field]['type'] == 'float'
-                    and isinstance(self._values[field], Decimal)):
-                # XML-RPC return Decimal for double
-                self._values[field] = float(self._values[field])
 
     def _default_get(self):
         'Set default values'
@@ -1142,9 +1152,8 @@ class Model(object):
             values.update(self._on_change_args(on_change))
         if values:
             context = self._context
-            changes = getattr(self._proxy, 'on_change')(values, names, context)
-            for change in changes:
-                self._set_on_change(change)
+            change = getattr(self._proxy, 'on_change')(values, names, context)
+            self._set_on_change(change)
 
         values = {}
         fieldnames = set(names)
@@ -1160,24 +1169,30 @@ class Model(object):
                 later.add(field)
                 continue
             to_change.add(field)
-            values.update(self._on_change_args(on_change_with))
+            values.update(self._on_change_args(on_change_with + [field]))
         if to_change:
             context = self._context
             changes = getattr(self._proxy, 'on_change_with')(values,
                 list(to_change), context)
             self._set_on_change(changes)
-        for field in later:
-            on_change_with = self._fields[field]['on_change_with']
-            values = self._on_change_args(on_change_with)
+        if later:
+            values = {}
+            for field in later:
+                on_change_with = self._fields[field]['on_change_with']
+                values.update(self._on_change_args(on_change_with + [field]))
             context = self._context
-            result = getattr(self._proxy, 'on_change_with_%s' % field)(values,
-                    context)
-            self._on_change_set(field, result)
+            changes = getattr(self._proxy, 'on_change_with')(
+                values, list(later), context)
+            self._set_on_change(changes)
 
         if self._parent:
             self._parent._changed.add(self._parent_field_name)
             self._parent._on_change([self._parent_field_name])
 
+    def notifications(self):
+        values = self._get_on_change_values()
+        return getattr(self._proxy, 'on_change_notify')(values, self._context)
+
 
 class Wizard(object):
     'Wizard class for Tryton wizards'
@@ -1237,7 +1252,10 @@ class Wizard(object):
                 view = result['view']
                 self.form = Model.get(
                     view['fields_view']['model'], self._config)()
-                self.form._default_set(view['defaults'])
+                if 'defaults' in view:
+                    self.form._default_set(view['defaults'])
+                if 'values' in view:
+                    self.form._values.update(view['values'])
                 self.states = [b['state'] for b in view['buttons']]
                 self.form_state = view['state']
             else:
@@ -1247,7 +1265,7 @@ class Wizard(object):
             for action in result.get('actions', []):
                 proteus_action = _convert_action(
                     *action, context=self._context, config=self._config)
-                if proteus_action:
+                if proteus_action is not None:
                     self.actions.append(proteus_action)
 
             if 'view' in result:
@@ -1293,8 +1311,16 @@ class Report(object):
 def _convert_action(action, data=None, context=None, config=None):
     if config is None:
         config = proteus.config.get_config()
+    records = None
     if data is None:
         data = {}
+    elif isinstance(data, (list, tuple)):
+        records = data
+        data = {
+            'model': records[0].__class__.__name__,
+            'id': records[0].id,
+            'ids': [r.id for r in records],
+            }
     else:
         data = data.copy()
 
@@ -1343,7 +1369,9 @@ def _convert_action(action, data=None, context=None, config=None):
             'config': config,
             'context': context,
             }
-        if 'model' in data:
+        if records is not None:
+            kwargs['models'] = records
+        elif 'model' in data:
             Model_ = Model.get(data['model'], config)
             config = Model_._config
             with config.reset_context(), config.set_context(context):
